/* GGL - GPUL Game Library
 * Copyright (C) 1998 G.P.U.L. (Group of programmers and users of Linux)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#ifndef __GGLFLI_H__
#define __GGLFLI_H__

#include <glib.h>
#include "gglvideo.h"

#define GGL_FLI(fli) (GGL_CAST (fli, GglFli*, GGLFLI))

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef struct _GglFli GglFli;


struct _GglFli{
  GglVideo video;
  int id;

  int fpt;              /* Frames per video tick - Speed measure */

  int fsize;            /* File size - Internal use */
  int offset;           /* File offset - Internal use */

};

/* Constructors */
GglFli*	        ggl_fli_new		();

/* Init all the internal structures */
void		ggl_fli_init		(GglFli *fli);

/* File Methods */
void		ggl_fli_open		(GglFli *fli,
					 char *fname);
void		ggl_fli_close		(GglFli *fli);
void		ggl_fli_attach		(GglFli *fli,
					 FILE *file_id);
void		ggl_fli_release		(GglFli *fli);
/* Display methods */
void		ggl_fli_attach_screen	(GglFli *fli,
					 GglScreen *screen);
void		ggl_fli_release_screen 	(GglFli *fli);

/* Play methods */
void		ggl_fli_play		(GglFli *fli);
void		ggl_fli_play_segment	(GglFli *fli,
					 unsigned long frame_ini,
					 unsigned long frame_fin);

/* Frame selection methods */
void		ggl_fli_rewind		(GglFli *fli);
void		ggl_fli_go_to		(GglFli *fli,
					 unsigned long frame);
void		ggl_fli_next		(GglFli *fli);
void		ggl_fli_previous	(GglFli *fli);

/* Single frame draw */
void		ggl_fli_draw_frame	(GglFli *fli);

/* Property methods */
unsigned long	ggl_fli_frames		(GglFli *fli);
char		ggl_fli_end		(GglFli *fli);
unsigned long	ggl_fli_current		(GglFli *fli);
int		ggl_fli_size_h		(GglFli *fli);
int		ggl_fli_size_v		(GglFli *fli);

#ifdef __cplusplus
      }
#endif /* __cplusplus */

#endif /* __GGLVIDEO_H__ */

