/* GGL - GPUL Game Library
 * Copyright (C) 1998 G.P.U.L. (Group of programmers and users of Linux)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __GGLLAYER_H__
#define __GGLLAYER_H__

#include <glib.h>
#include <ggl/gglgraphic.h>

#define GGL_LAYER(layer) (GGL_CAST(layer,GglLayer*,GGLLAYER))

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef struct _GglLayer GglLayer;

struct _GglLayer{
    GglBase base;
    int id;
    GList *graphics;  /* List of graphics that belong to this layer */
    int scroll_x, scroll_y; /* Each layer can have it's independent scroll */
    GList *deleted; /* Used to know objects deleted that must be redrawn */
    void (*draw)  (GglLayer *layer, GglVisual *visual, GList *modified);
    void (*change)(GglLayer *layer, GList *modified);
};

GglLayer*	ggl_layer_new		(void);

void		ggl_layer_init		(GglLayer *layer);

void		ggl_layer_dispose	(GglLayer *layer);

void		ggl_layer_finalize	(GglLayer *layer);

void		ggl_layer_draw          (GglLayer *layer,
					 GglVisual *visual,
					 GList *modified);

void            ggl_layer_change        (GglLayer *layer,
					 GList* *modified);

void            ggl_layer_add           (GglLayer *layer,
					 GglGraphic *gr);

void            ggl_layer_remove        (GglLayer *layer,
					 GglGraphic *gr);

#ifdef __cplusplus
      }
#endif /* __cplusplus */

#endif /* __GGLLAYER_H__ */

