/* GGL - GPUL Game Library
 * Copyright (C) 1998 G.P.U.L. (Group of programmers and users of Linux)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __GGLOBSERVER_H__
#define __GGLOBSERVER_H__

#include <glib.h>
#include <ggl/gglbase.h>
#include <ggl/ggltimeout.h>
#include <ggl/gglvisual.h>

#define GGL_OBSERVER(observer) (GGL_CAST(observer,GglObserver*,GGLOBSERVER))

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef struct _GglObserver GglObserver;

struct _GglObserver{
  GglBase base;
  int id;

  void (*update_axis)(GglObserver *observer,
                      GglDeviceType type,   
		      int device_number, 
		      int axis,         
		      int value);
  void (*update_button)(GglObserver *observer,
  			GglDeviceType type,
  			int device_number, 
  			int button,       
  			int value);
  			
  void (*update_collision)(GglObserver *observer,
  			   GglDeviceType type,   
  			   void* first,
  			   void* second,
  			   float angle);
  			   
  void (*update_finish)(GglObserver *observer,
  			GglDeviceType type,   
  			int device_number, 
  			int action);        			
};

/* Constructors */
GglObserver*	ggl_observer_new		();
void		ggl_observer_init		(GglObserver *observer);

void		ggl_observer_dispose		(GglObserver *observer);

void		ggl_observer_finalize		(GglObserver *observer);

void		ggl_observer_update_axis	(GglObserver *observer,
						 GglDeviceType type,
						 int device_number,
						 int axis,
						 int value);
						 
void		ggl_observer_update_button	(GglObserver *observer,
						 GglDeviceType type,
						 int device_number,
						 int button,
						 int value);
						 
/* This method may change, be careful */
void		ggl_observer_update_collision	(GglObserver *observer,
						 GglDeviceType type,
						 void* first,
						 void* second,
						 float angle);
						 
void		ggl_observer_update_finish	(GglObserver *observer,
						 GglDeviceType type,
						 int device_number,
						 int action);

#ifdef __cplusplus
      }
#endif /* __cplusplus */

#endif /* __GGLOBSERVER_H__ */
