/* GGL - GPUL Game Library
 * Copyright (C) 1998 G.P.U.L. (Group of programmers and users of Linux)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __GGLUDPSOCKET_H__
#define __GGLUDPSOCKET_H__

#include <ggl/gglbase.h>
#include <sys/types.h>
#include <sys/socket.h>

#define GGL_UDPSOCKET(udp_socket) (GGL_CAST(udp_socket,GglUdpSocket*,GGLUDPSOCKET))

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef struct _GglUdpSocket GglUdpSocket;

struct _GglUdpSocket{
  GglBase base;         /* Base class */
  int id;               /* Class Identifier */

  char *local_IP;     	/* Local IP of this machine */
  int *local_port;	/* Local port */
  char *remote_IP;	/* Remote IP */
  int *remote_port;	/* Remote port */
  int local_socket;	/* Raw UNIX file descriptor for the socket */
  struct sockaddr *remote_socket; /* Raw UNIX structure describing the remote socket */
};

/* Constructors */
GglUdpSocket *	ggl_udp_socket_new	(int	local_port,
					 char *	remote_IP,
					 int	remote_port);
void	ggl_udp_socket_init		(GglUdpSocket *	udp_socket);
char *	ggl_udp_socket_local_IP		(char *	ip);
int	ggl_udp_socket_local_port       (void);
char *	ggl_udp_socket_remote_IP	(char *	ip);
int	ggl_udp_socket_remote_port	(void);
int	ggl_udp_socket_local_socket	(void);
struct sockaddr *	ggl_udp_socket_remote_socket	(struct sockaddr *	remote_socket);
void	ggl_udp_socket_finalize		(GglUdpSocket *	udp_socket);
void	ggl_udp_socket_dispose		(GglUdpSocket *	udp_socket);

/* Private functions */
					
#ifdef __cplusplus
      }
#endif /* __cplusplus */

#endif /* __GGLUDPSOCKET_H__ */

