/* GGL - GPUL Game Library
 * Copyright (C) 1998 G.P.U.L. (Group of programmers and users of Linux)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>

#include <ggl/gglarbiter.h>
#include <ggl/gglsprite.h>

void gglarbiter_ticker(GglArbiter *arbiter);

typedef struct {
    GglSprite *first,*second;
    float angle;
}Collision;

void ggl_arbiter_init(GglArbiter *arbiter)
{
    ggl_device_init(GGL_DEVICE(arbiter));
    arbiter-> id      = GGLARBITER;
    arbiter-> sprites = NULL;
    arbiter-> deleted = NULL;
    GGL_DEVICE(arbiter)->ticker=(void(*)(GglDevice*))gglarbiter_ticker;
}

GglArbiter* ggl_arbiter_new()
{
    GglArbiter *arbiter=(GglArbiter *)malloc(sizeof(GglArbiter));
    ggl_base_set_size(GGL_BASE(arbiter),sizeof(GglArbiter));
    
    ggl_arbiter_init(arbiter);
    
    return arbiter;
}

void ggl_arbiter_finalize(GglArbiter *arbiter)
{
    ggl_device_finalize(GGL_DEVICE(arbiter));
    g_list_free(arbiter->sprites);
    arbiter->sprites=NULL;
}

void ggl_arbiter_dispose(GglArbiter *arbiter)
{
    ggl_arbiter_finalize(arbiter);
    free(arbiter);
}

void ggl_arbiter_listen (GglArbiter *arbiter, GglSprite *sprite)
{
    arbiter->sprites = g_list_append(arbiter->sprites,sprite);
}

void ggl_arbiter_ignore (GglArbiter *arbiter, GglSprite *sprite)
{
    arbiter->deleted = g_list_append(arbiter->deleted,sprite);
}

void gglarbiter_collision_notify(GglDeviceItem *devitem, Collision *col)
{
    if (devitem == NULL) g_warning("gglkeydev_button_notify: NULL pointer\n");
    if (devitem->mask & GGL_EVMASK_ANY_COLLISION)
	ggl_observer_update_collision(devitem->observer,GGL_ARBITER,col->first,
				      col->second,
				      col->angle);
    else if (devitem->mask & GGL_EVMASK_OWN_COLLISION)
    {
	if (devitem->observer == GGL_OBSERVER(col->first)
	    || devitem->observer == GGL_OBSERVER(col->second))
	    ggl_observer_update_collision(devitem->observer,GGL_ARBITER,col->first,
					  col->second,
					  col->angle);
	    
    }
}

void gglarbiter_ticker(GglArbiter *arbiter)
{
    Collision col;
    GList *node;
    int was_deleted=0; /* node was deleted. */
    for (node=arbiter->sprites; node;)
    {
	GList *intern_node;
	float angle;
	GglSprite *sprite=(GglSprite*)node->data;
	for (intern_node=node->next; intern_node; )
	{
	    int int_was_deleted=0;
	    GglSprite *intern_sprite=(GglSprite*)intern_node->data;
	    if (ggl_sprite_collision(sprite,intern_sprite,&angle))
	    {
		GList *item;
		col.first=sprite;
		col.second=intern_sprite;
		col.angle=angle;
		ggl_device_foreach_observer(GGL_DEVICE(arbiter),
					    (GglDevCallBack)gglarbiter_collision_notify,
					    &col);
		/* Processing deleted sprites... */
		for (item=arbiter->deleted; item; item=item->next)
		{
		    GglSprite *to_delete=(GglSprite*)item->data;
		    if (to_delete==intern_sprite)
		    {
			int_was_deleted=1;
			if (intern_node)
			{
			    intern_node=intern_node->next;
			    intern_sprite=(GglSprite*)intern_node->data;
			}
		    }
		    if (to_delete==sprite)
		    {
			was_deleted=1;
			if (node)
			{
			    node=node->next;
			}
			intern_node=NULL;
		    }
                    g_list_remove(arbiter->sprites,to_delete);
		}
		g_list_free(arbiter->deleted);
		arbiter->deleted=NULL;
	    }
	    if (!int_was_deleted)
	    {
		if (intern_node)
		{
		    intern_node=intern_node->next;
		}
	    }
	    int_was_deleted=0;
	}
	if (!was_deleted)
	{
	    if (node)
	    {
		node=node->next;
	    }
	}
	was_deleted=0;
    }
}
