/* GGL - GPUL Game Library
 * Copyright (C) 1998 G.P.U.L. (Group of programmers and users of Linux)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <ggl/gglanim.h>
#include <stdlib.h>
#include "stdio.h"

/************************************************************************/
void ggl_anim_init (GglAnim *anim)
{
    anim-> id=GGLANIM;
    anim-> speed=24;
    anim-> frames=NULL;
    anim-> cur_frame=NULL;
    anim-> num_frames=0;
}

/************************************************************************/
GglAnim* ggl_anim_new (void)
{
    GglAnim *anim=(GglAnim *)malloc(sizeof(GglAnim));
    ggl_base_set_size(GGL_BASE(anim),sizeof(GglAnim));

    ggl_anim_init(anim);

    return anim;
}

/************************************************************************/
void ggl_anim_finalize (GglAnim *anim)
{
    g_list_free(anim->frames);
}

/************************************************************************/
void ggl_anim_dispose (GglAnim *anim)
{
    ggl_anim_finalize(anim);
    free(anim);
}

/************************************************************************/
void ggl_anim_append (GglAnim *anim, GglFrame *frame)
{
    anim->frames=g_list_append(anim->frames,frame);
    anim->num_frames++;
    if (anim->cur_frame==NULL)
	anim->cur_frame=ggl_descriptor_new(anim->frames);
}

/************************************************************************/
void ggl_anim_prepend (GglAnim *anim, GglFrame *frame)
{
    anim->frames=g_list_prepend(anim->frames,frame);
    anim->num_frames++;
    if (anim->cur_frame==NULL)
	anim->cur_frame=ggl_descriptor_new(anim->frames);
}

/************************************************************************/
void ggl_anim_remove (GglAnim *anim, GglFrame *frame)
{
    anim->frames=g_list_remove(anim->frames,frame);
    anim->num_frames--;
    ggl_descriptor_dispose(anim->cur_frame);
    if (anim->frames!=NULL)
	anim->cur_frame=ggl_descriptor_new(anim->frames);
    else
	anim->cur_frame=NULL;
}

/************************************************************************/
void ggl_anim_insert (GglAnim *anim, GglFrame *frame, int position)
{
    anim->frames=g_list_insert(anim->frames,frame,position);
    anim->num_frames++;
    if (anim->cur_frame==NULL)
	anim->cur_frame=ggl_descriptor_new(anim->frames);
}

/* Main methods for drawing an anim */
/************************************************************************/
void ggl_anim_draw (GglAnim *anim, GglVisual *visual, int x, int y)
{
    GglFrame *frame;
    if (anim->frames!=NULL)
    {
	frame=(GglFrame*)ggl_descriptor_data(anim->cur_frame);
	ggl_frame_draw(frame,visual,x,y);
    }
}

/* Other methods */				 
/************************************************************************/
void ggl_anim_set_next_frame (GglAnim *anim)
{
    if (anim->num_frames!=0)
    {
	if (ggl_descriptor_inc(anim->cur_frame)==-1)
	{
	    ggl_descriptor_dispose(anim->cur_frame);
	    anim->cur_frame=ggl_descriptor_new(anim->frames);
	}
    }
}

/************************************************************************/
void ggl_anim_set_frame (GglAnim *anim, GglDescriptor *descriptor)
{
    anim->cur_frame=descriptor;
}

/************************************************************************/
void ggl_anim_set_frame_nth (GglAnim *anim, int frame)
{
    if (frame<anim->num_frames)
    {
	ggl_descriptor_dispose(anim->cur_frame);
	anim->cur_frame=ggl_descriptor_new(g_list_nth(anim->frames,frame));
    }
    else printf("WARNING: ggl_anim_set_frame\n");
}

/************************************************************************/
GglDescriptor* ggl_anim_get_frame (GglAnim *anim)
{
    return anim->cur_frame;
}

/************************************************************************/
void ggl_anim_draw_frame (GglAnim *anim, int frame, GglVisual *visual, int x, int y)
{
    GglFrame *gglframe=(GglFrame*)ggl_descriptor_data(anim->cur_frame);
    ggl_frame_draw(gglframe,visual,x,y);
}

/************************************************************************/
int ggl_anim_length (GglAnim *anim)
{
    return anim->num_frames;
}

/************************************************************************/
void ggl_anim_get_frame_size (GglAnim *anim, int frame, int *width, int *height)
{
    GglFrame *gglframe=(GglFrame*)ggl_descriptor_data(anim->cur_frame);
    ggl_frame_get_size(gglframe,width,height);
}

/************************************************************************/
void ggl_anim_cur_frame_size (GglAnim *anim, int *width, int *height)
{
    GglFrame *frame=(GglFrame*)ggl_descriptor_data(anim->cur_frame);
    ggl_frame_get_size(frame,width,height);
}

/************************************************************************/
GglAnim *ggl_anim_clone (GglAnim *anim)
{
    GglAnim *new_anim=(GglAnim *)malloc(sizeof(GglAnim));
    memcpy(new_anim,anim,sizeof(GglAnim));
    new_anim->cur_frame=ggl_descriptor_clone(anim->cur_frame);
    return new_anim;
}

/************************************************************************/
void ggl_anim_cur_frame_hook(GglAnim *anim, short *x, short *y)
{
    GglFrame *frame=(GglFrame*)ggl_descriptor_data(anim->cur_frame);
    ggl_frame_get_hook(frame,x,y);
}
