/* GGL - GPUL Game Library
 * Copyright (C) 1998 G.P.U.L. (Group of programmers and users of Linux)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <ggl/gglvisual.h>
#include <stdlib.h>

void ggl_visual_init_target(GglVisual *visual, char *target)
{

    visual-> id=GGLVISUAL;
    visual-> visual=ggiOpen(target,NULL);
    visual-> width=0;
    visual-> height=0;
    visual-> swidth=0;
    visual-> sheight=0;
    visual-> scrollx=0;
    visual-> scrolly=0;
    visual-> scrolled=0;
    visual-> depth=0;
    if (visual->visual == NULL)
    {
        printf("Error, ggiOpen failed\n");
        exit(1);
    }    
    else 
        {
        ggiAddFlags(visual->visual,GGIFLAG_ASYNC);
        }
}
GglVisual* ggl_visual_new_target(char *target)
{
    GglVisual *visual=(GglVisual *)malloc(sizeof(GglVisual));
    ggl_base_set_size(GGL_BASE(visual),sizeof(GglVisual));
    
    ggl_visual_init_target(visual,target);
    
    return visual;
}

void ggl_visual_init (GglVisual *visual)
{
ggl_visual_init_target(visual,NULL);
}

void ggl_visual_finalize (GglVisual *visual)
{
  ggiClose(visual->visual);
}

void ggl_visual_dispose (GglVisual *visual)
{
  ggl_visual_finalize(visual);
  free(visual);
}

GglVisual* ggl_visual_new()
{
    return ggl_visual_new_target(NULL);
}

int depth2ggi_graph_type(GglDepth depth)
{
    switch(depth){
     case GGL_1BIT:return GT_1BIT;
     case GGL_4BIT:return GT_4BIT;
     case GGL_8BIT:return GT_8BIT;
     case GGL_15BIT:return GT_15BIT;
     case GGL_16BIT:return GT_16BIT;
     case GGL_24BIT:return GT_24BIT;
     case GGL_32BIT:return GT_32BIT;
     default: return GGI_AUTO;
    }
}

int ggl_visual_set_mode(GglVisual *visual,int width,int height, int vwidth, int vheight,GglDepth depth, int frames)
{
    int error;
    ggi_mode gmode;

    visual->depth=depth;
    gmode.frames=frames;
    gmode.visible.x=width;
    gmode.visible.y=height;
    gmode.virt.x=vwidth;
    gmode.virt.y=vheight;
    gmode.graphtype=depth2ggi_graph_type(depth);
    gmode.dpp.x=1;
    gmode.dpp.y=1;
    error= ggiSetMode(visual->visual,&gmode);
    ggiGetMode(visual->visual,&gmode);
    visual->width=gmode.visible.x;
    visual->height=gmode.visible.y;
    visual->swidth=gmode.virt.x;
    visual->sheight=gmode.virt.y;
    visual->frames=gmode.frames;
    return error;
}

int ggl_visual_check_mode(GglVisual *visual,int width,int height, int vwidth, int vheight,GglDepth depth, int frames)
{
    int error;
    ggi_mode gmode;

    visual->depth=depth;
    gmode.frames=frames;
    gmode.visible.x=width;
    gmode.visible.y=height;
    gmode.virt.x=vwidth;
    gmode.virt.y=vheight;
    gmode.graphtype=depth2ggi_graph_type(depth);
    gmode.dpp.x=1;
    gmode.dpp.y=1;
    error= ggiCheckMode(visual->visual,&gmode);
    return error;
}
int ggl_visual_set_origin(GglVisual *visual,int x,int y)
{
    return ggiSetOrigin(visual->visual,x,y);
}

int ggl_visual_flush(GglVisual *visual)
{
return ggiFlush(visual->visual);
}

int ggl_visual_flush_region(GglVisual *visual, int x, int y, int w, int h)
{
return ggiFlushRegion(visual->visual,x,y,w,h);
}

void ggl_visual_set_palette(GglVisual *visual,GglPalette *palette)
{
  ggiSetPalette(visual->visual,palette->first, palette->number,palette->palette);
}

void ggl_visual_get_palette(GglVisual *visual,GglPalette *palette)
{
  ggiGetPalette(visual->visual,palette->first, palette->number,palette->palette);
}

/* GGI's version, doesn't run at all, but i don't know why... */
int ggl_visual_change_depth(GglVisual *visual,GglDepth target_depth)
{
    GglVisual *target;
    target=ggl_visual_new_target("display-memory");
    if (ggl_visual_set_mode(target,visual->width,visual->height,
                        visual->swidth,visual->sheight,
                        target_depth,visual->frames)) return -1;
    if (ggiCrossBlit(visual->visual,0,0,visual->swidth,visual->sheight,target->visual,0,0)) return -1;
    ggiClose(visual->visual);
    visual->visual=target->visual;
    free(target);
    return 0;
}

int ggl_visual_get_bits_size(GglVisual *visual)
{
return ggiGetPixelFormat(visual->visual)->size;
}

int ggl_visual_set_disp_frame (GglVisual *visual,int frame)
{
    return ggiSetDisplayFrame(visual->visual,frame);
}
int ggl_visual_set_read_frame (GglVisual *visual,int frame)
{
    return ggiSetReadFrame(visual->visual,frame);
}

int ggl_visual_set_write_frame (GglVisual *visual,int frame)
{
    return ggiSetWriteFrame(visual->visual,frame);
}

int ggl_visual_get_disp_frame (GglVisual *visual)
{
    return ggiGetDisplayFrame(visual->visual);
}

int ggl_visual_get_read_frame (GglVisual *visual)
{
    return ggiGetReadFrame(visual->visual);
}

int ggl_visual_get_write_frame (GglVisual *visual)
{
    return ggiGetWriteFrame(visual->visual);
}

void* ggl_visual_get_read_pointer (GglVisual *visual)
{
    return ggiDBGetBuffer(visual->visual,ggl_visual_get_read_frame(visual))->read;
}

void* ggl_visual_get_write_pointer (GglVisual *visual)
{
    return ggiDBGetBuffer(visual->visual,ggl_visual_get_write_frame(visual))->write;
}


void *ggl_visual_get_read_pointer_nth(GglVisual *visual, int frame)
{
return ggiDBGetBuffer(visual->visual,frame)->read;
}

void *ggl_visual_get_write_pointer_nth(GglVisual *visual, int frame)
{
return ggiDBGetBuffer(visual->visual,frame)->write;
}
