#!/usr/bin/perl
#  Copyright (C) 1999  G.P.U.L. (Group of programmers and users of Linux)
#
#  This script is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

my $DEBUG_MODE=""; # set to yes to debug

sub parse
{
    my $file=shift(@_);
    my $prefix=shift(@_);
    open(FILE,"<$file");
    my @header_list=();    
    while (<FILE>)
    {
        my $linea=$_;
        my $prefijo="";
        $linea =~ /^[ \t]*$prefix[ \t]*(.*)/;
        $linea=$1;
        # Sacamos los prefijos
        while ($linea =~ /^[ \t]*(struct) (.*)/
                      || $linea =~ /^[ \t]*(unsigned) (.*)/
                      || $linea =~ /^[ \t]*(signed) (.*)/
                      || $linea =~ /^[ \t]*(short) (.*)/
                      || $linea =~ /^[ \t]*(long) (.*)/)
        {
            if ($prefijo) {$prefijo="$prefijo.$1";}
            else {$prefijo=$1; }
            $linea=$2;
        }
        # Ahora sacamos hasta el parentesis
        if ($linea =~ /^[ \t]*($first[^ \t\*]+)[ \t]*(\*?)[ \t]*(ggl_[^ \t]*)[ \t]*\((.*)$/)
        {
            my $return="";
            if ($prefijo) {$return="$prefijo.$1";}
            else {$return=$1; }
            if ($2) {$return="$return$2";}
            my $nombre=$3;
            if ($DEBUG_MODE){print "[$return]-[$nombre]\n";}
            my @function=($return,$nombre);
            $linea=$4;
            $prefijo="";
            # Vamos sacando prefijos si los hay
            while ($linea =~ /^[ \t]*(struct) (.*)/
                          || $linea =~ /^[ \t]*(unsigned) (.*)/
                          || $linea =~ /^[ \t]*(signed) (.*)/
                          || $linea =~ /^[ \t]*(short) (.*)/
                          || $linea =~ /^[ \t]*(long) (.*)/)
            {
                if ($prefijo) {$prefijo="$prefijo.$1";}
                else {$prefijo=$1; }
                $linea=$2;
            }
            while ($linea =~ /^[ \t]*([^ \t\*]*)[ \t]*(\*?)[ \t]*([^ \t\*]*)[ \t]*,[ \t]*(.*)$/)
            {
                my $tipo="";
                if ($prefijo) {$tipo="$prefijo.$1";}
                else {$tipo=$1; }
                if ($2) {$tipo="$tipo$2";}
                my $nombre=$3;
                if ($DEBUG_MODE){print "***>[$tipo]-[$nombre]\n";}
                push @function,$tipo;
                push @function,$nombre;
                $linea=$4 || <FILE>;
                if ($linea =~ /^[ \t]*\/\*.*\*\/[ \t]*(.*)/)
                {
                    $linea=$1 || <FILE>;
                }
                $prefijo="";
                #Sacamos los prefijos de lo siguiente que venga
                while ($linea =~ /^[ \t]*(struct) (.*)/
                              || $linea =~ /^[ \t]*(unsigned) (.*)/
                              || $linea =~ /^[ \t]*(signed) (.*)/
                              || $linea =~ /^[ \t]*(short) (.*)/
                              || $linea =~ /^[ \t]*(long) (.*)/)
                {
                    if ($prefijo) {$prefijo="$prefijo.$1";}
                    else {$prefijo=$1; }
                    $linea=$2;
                }
            }
            # La ltima linea de la funcion se procesa a parte, porque si no
            # si lo hacemos con la funcion de antes, siempre leeremos una
            # linea de mas y nos cargaremos la siguiente funcion
            if ($linea =~ /^[ \t]*([^ \t\*]*)[ \t]*(\*?)[ \t]*([^ \t\*]*)[ \t]*\)(.*)$/)
            {
                if ($prefijo) {$tipo="$prefijo.$1";}
                else {$tipo=$1; }
                if ($2) {$tipo="$tipo$2";}
                my $nombre=$3;
                if ($tipo ne "void")
                {
                    if ($DEBUG_MODE){print "--->[$tipo]-[$nombre]\n";}
                    push @function,$tipo;
                    push @function,$nombre;
                }
            }
            # Metemos en header_list la funcion, con un \n para poder
            # manejar luego la estructura mas comodamente.
            push @header_list, "@function\n";
        }
    }
    close(FILE);
    return @header_list;
}

#####################################################################

if ($#ARGV == -1)
  {
    print "CHECK.PL: The documentation update control script of ggl.\n";
    print "Usage: ";
    print "check.pl <class_name>.h <class_name>.gglh\n";
    print "\n";
    exit;
  }

print "CHECK.PL v1.0 -* $ARGV[0],$ARGV[1] *-\n";

my $class_h=$ARGV[0];
my $class_gglh=$ARGV[1];

@header=parse("$class_h","");
@docum=parse("$class_gglh","%format");

#####################################################################

sub busca_docum
{    
    my @funcion=split(' ',shift(@_));
    my $tipo=shift(@funcion);
    my $nombre=shift(@funcion);
    my @errlist=();
    my $found="";
    foreach my $d_linea (@docum){
        my @d_func=split(' ',$d_linea);
        my $d_tipo=shift(@d_func);
        my $d_nombre=shift(@d_func);
        if ($nombre eq $d_nombre)            
        {
            $found ="SI";
            if ($tipo ne $d_tipo)
            {
                printf("ERROR: return at <$nombre>\n");
            }
            my $i=0;
            while (@funcion && @d_func)
            {
                $i++;
                my $tipo=shift(@funcion);
                my $var=shift(@funcion);
                my $d_tipo=shift(@d_func);
                my $d_var=shift(@d_func);
                if ($tipo ne $d_tipo)
                {
                    printf("ERROR: arg $i at <$nombre> ($tipo != $d_tipo)\n");
                }
                if ($var ne $d_var)
                {
                    printf("WARNING: arg $i at <$nombre> ($var != $var)\n");
                }
            }
            if (@d_func)
            {
                printf("ERROR: too many args at <$nombre> (@d_func)\n");
            }
            if (@funcion)
            {
                printf("ERROR: too few args at <$nombre> (@funcion)\n");
            }
        }
    }
    if (not $found)
    {
        $_=$nombre;
        SWITCH:{
            if (/^ggl_.*_new/) { printf("FATAL ERROR: Method <<NEW>> undocumented!\n"); }
            elsif (/^ggl_.*_init/) { printf("DON'T WORRY: Method init undocumented...\n"); }
            elsif (/^ggl_.*_dispose/) { printf("DON'T WORRY: Method dispose undocumented...\n"); }
            elsif (/^ggl_.*_finalize/) { printf("DON'T WORRY: Method finalize undocumented...\n"); }
            else { printf("ERROR: <$nombre> not found\n"); }
        }
    }
}

foreach my $koso (@header){
    busca_docum($koso);
}
foreach my $koso (@docum){
    my @funcion=split(' ',$koso);
    my $nombre=$funcion[1];
    $found="";
    foreach my $h_linea (@header){
        my @h_func=split(' ',$h_linea);
        my $h_nombre=$h_func[1];
        if ($nombre eq $h_nombre) {$found="SI!";}
    }
    if (not $found)
    {
        printf("ERROR: <$nombre> is obsolete\n");
    }
}
